import Router, { useRouter } from 'next/router';
import { useMutation, useQuery, useQueryClient } from 'react-query';
import { toast } from 'react-toastify';
import { useTranslation } from 'next-i18next';
import { Routes } from '@/config/routes';
import { API_ENDPOINTS } from './client/api-endpoints';
import {
  Author,
  AuthorPaginator,
  AuthorQueryOptions,
  GetParams,
} from '@/types';
import { mapPaginatorData } from '@/utils/data-mappers';
import { AuthorClient } from './client/author';
import { Config } from '@/config';

export const useCreateAuthorMutation = () => {
  const queryClient = useQueryClient();
  const { t } = useTranslation();
  const router = useRouter();

  return useMutation(AuthorClient.create, {
    onSuccess: async () => {
      const generateRedirectUrl = router.query.shop
        ? `/${router.query.shop}${Routes.author.list}`
        : Routes.author.list;
      await Router.push(generateRedirectUrl, undefined, {
        locale: Config.defaultLanguage,
      });
      toast.success(t('common:successfully-created'));
    },
    // Always refetch after error or success:
    onSettled: () => {
      queryClient.invalidateQueries(API_ENDPOINTS.AUTHORS);
    },
  });
};

export const useDeleteAuthorMutation = () => {
  const queryClient = useQueryClient();
  const { t } = useTranslation();

  return useMutation(AuthorClient.delete, {
    onSuccess: () => {
      toast.success(t('common:successfully-deleted'));
    },
    // Always refetch after error or success:
    onSettled: () => {
      queryClient.invalidateQueries(API_ENDPOINTS.AUTHORS);
    },
  });
};

export const useUpdateAuthorMutation = () => {
  const { t } = useTranslation();
  const queryClient = useQueryClient();
  return useMutation(AuthorClient.update, {
    onSuccess: () => {
      toast.success(t('common:successfully-updated'));
    },
    // Always refetch after error or success:
    onSettled: () => {
      queryClient.invalidateQueries(API_ENDPOINTS.AUTHORS);
    },
  });
};

export const useAuthorQuery = ({ slug, language }: GetParams) => {
  const { data, error, isLoading } = useQuery<Author, Error>(
    [API_ENDPOINTS.AUTHORS, { slug, language }],
    () => AuthorClient.get({ slug, language })
  );

  return {
    author: data,
    error,
    loading: isLoading,
  };
};

export const useAuthorsQuery = (options: Partial<AuthorQueryOptions>) => {
  const { data, error, isLoading } = useQuery<AuthorPaginator, Error>(
    [API_ENDPOINTS.AUTHORS, options],
    ({ queryKey, pageParam }) =>
      AuthorClient.paginated(Object.assign({}, queryKey[1], pageParam)),
    {
      keepPreviousData: true,
    }
  );

  return {
    authors: data?.data ?? [],
    paginatorInfo: mapPaginatorData(data),
    error,
    loading: isLoading,
  };
};
